# SPDX-FileCopyrightText: 2026 Martin Kirchgessner
#
# SPDX-License-Identifier: AGPL-3.0-or-later

"""
Radiotomate DB entities, as SQLAlchemy mapped classes.

Those classes are used by the SQLAlchemy ORM to map the database tables to
Python objects. They can implement methods, but should not ``commit`` (at most,
``flush``).
"""

from radiotomate.models.autodj_slot import AutoDJSlot
from radiotomate.models.cart import Cart
from radiotomate.models.metadata_log import MetadataLog
from radiotomate.models.session import Session
from radiotomate.models.setting import Setting
from radiotomate.models.sound import Sound
from radiotomate.models.user import LoggedOutUser, User

__all__ = [
    "AutoDJSlot",
    "Cart",
    "LoggedOutUser",
    "MetadataLog",
    "Session",
    "Setting",
    "Sound",
    "User",
]
